"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = __importStar(require("@oclif/command"));
const promise_1 = __importDefault(require("simple-git/promise"));
const Context = __importStar(require("../utils/context"));
const Output = __importStar(require("../utils/output"));
const publish_1 = require("../utils/publish");
const Rel = __importStar(require("../utils/release"));
class Stable extends command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Stable);
            const git = promise_1.default();
            const show = createShowers({ json: flags.json });
            const check = createValidators({ json: flags.json });
            const ctx = yield Context.scan({
                overrides: {
                    trunk: flags.trunk || null,
                },
            });
            if (!check.isTrunk(ctx))
                return;
            if (!check.branchSynced(ctx))
                return;
            if (!check.notAlreadyStableReleased(ctx))
                return;
            const release = Rel.getNextStable(ctx.series);
            if (Rel.isNoReleaseReason(release)) {
                return show.noReleaseNeeded(ctx, release);
            }
            if (flags['dry-run']) {
                return show.dryRun(ctx, release);
            }
            yield publish_1.publish({
                version: release.version.version,
                distTag: 'latest',
                additiomalDistTags: ['next'],
            }, { skipNPM: flags['skip-npm'] });
            // force update so the tag moves to a new commit
            yield git.raw(['tag', '-f', 'latest']);
            yield git.raw(['tag', '-f', 'next']);
            // https://stackoverflow.com/questions/8044583/how-can-i-move-a-tag-on-a-git-branch-to-a-different-commit
            yield git.raw(['push', 'origin', ':refs/tags/latest', ':refs/tags/next']);
            yield git.raw(['push', '--tags']);
            console.log('updated git-tags "latest" "next"');
        });
    }
}
exports.Stable = Stable;
Stable.flags = {
    trunk: command_1.flags.string({
        default: '',
        description: 'State which branch is trunk. Defaults to honuring the "base" branch setting in the GitHub repo settings.',
    }),
    'dry-run': command_1.flags.boolean({
        default: false,
        description: 'output what the next version would be if released now',
        char: 'd',
    }),
    json: command_1.flags.boolean({
        default: false,
        description: 'format output as JSON',
        char: 'j',
    }),
    // todo test skip-npm
    'skip-npm': command_1.flags.boolean({
        default: false,
        description: 'skip the step of publishing the package to npm',
    }),
};
function createShowers(opts) {
    function noReleaseNeeded(ctx, reason) {
        if (reason === 'no_meaningful_change') {
            Output.outputException('only_chore_like_changes', // todo replace this with reason forward
            'The release you attempting only contains chore commits which means no release is needed.', {
                json: opts.json,
                context: {
                    commits: ctx.series.commitsInNextStable.map(c => c.message),
                },
            });
        }
        else if (reason === 'empty_series') {
            Output.outputException(reason, 'There are no commits to release since the last stable.', {
                json: opts.json,
                context: {
                    commits: ctx.series.commitsInNextStable.map(c => c.message),
                },
            });
        }
    }
    function dryRun(ctx, release) {
        Output.outputOk('dry_run', Object.assign(Object.assign({}, release), { commits: ctx.series.commitsInNextStable }));
    }
    return {
        noReleaseNeeded,
        dryRun,
    };
}
function createValidators(opts) {
    function branchSynced(ctx) {
        if (ctx.currentBranch.syncStatus !== 'synced') {
            Output.outputException('branch_not_synced_with_remote', 'You are attempting a stable release but your trunk (aka. master/base branch) is not synced with the remote.', {
                json: opts.json,
                context: {
                    syncStatus: ctx.currentBranch.syncStatus,
                    sha: ctx.series.current.sha,
                },
            });
            return false;
        }
        return true;
    }
    function isTrunk(ctx) {
        if (!ctx.currentBranch.isTrunk) {
            Output.outputException('must_be_on_trunk', 'You are attempting a stable release but you are not on trunk (aka. master/base branch)', {
                json: opts.json,
                context: {
                    branch: ctx.currentBranch.name,
                    sha: ctx.series.current.sha,
                },
            });
            return false;
        }
        return true;
    }
    function notAlreadyStableReleased(ctx) {
        if (ctx.series.current.releases.stable) {
            Output.outputException('commit_already_has_stable_release', 'You are attempting a stable release on a commit that already has a stable release.', {
                json: opts.json,
                context: {
                    version: ctx.series.current.releases.stable.version,
                    sha: ctx.series.current.sha,
                },
            });
            return false;
        }
        return true;
    }
    return {
        branchSynced,
        isTrunk,
        notAlreadyStableReleased,
    };
}
