"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const rest_1 = require("@octokit/rest");
const promise_1 = __importDefault(require("simple-git/promise"));
const Git = __importStar(require("../lib/git"));
const PackageJson = __importStar(require("../lib/package-json"));
const Rel = __importStar(require("./release"));
function scan(opts) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        // Build up instances
        const octoOps = {};
        if (process.env.GITHUB_TOKEN)
            octoOps.auth = process.env.GITHUB_TOKEN;
        const octokit = new rest_1.Octokit(octoOps);
        const git = promise_1.default();
        // Generally required information
        const githubRepoAddress = yield Git.parseGithubRepoInfoFromGitConfig();
        // Get the trunk branch either from a given override or by default from the
        // GitHub repo settings.
        let trunkBranch;
        if ((_b = (_a = opts) === null || _a === void 0 ? void 0 : _a.overrides) === null || _b === void 0 ? void 0 : _b.trunk) {
            trunkBranch = opts.overrides.trunk;
        }
        else {
            const githubRepo = yield octokit.repos.get({
                owner: githubRepoAddress.owner,
                repo: githubRepoAddress.name,
            });
            trunkBranch = githubRepo.data.default_branch;
        }
        // Get the PRs if any
        const branchesSummary = yield git.branch({});
        const prs = (yield octokit.pulls.list({
            owner: githubRepoAddress.owner,
            repo: githubRepoAddress.name,
            head: `${githubRepoAddress.owner}:${branchesSummary.current}`,
            state: 'all',
        })).data;
        // Github only permits one open PR per branch name. If there is an open one
        // then we can discard any others as inconsequential. If none are open, and
        // there are multiple, we should not discard them, as they may be of interest
        // to the user to debug an exception, e.g. attempting a pr release without a
        // pr open: then we can say "Hey, you have no pr open, but oddly we did find
        // these past PRs...?"
        const prsByState = {
            open: null,
            closed: [],
        };
        for (const pr of prs) {
            if (pr.state === 'open') {
                prsByState.open = { number: pr.number, title: pr.title };
            }
            else {
                prsByState.closed.push({ number: pr.number, title: pr.title });
            }
        }
        // get the branch sync status
        const syncStatus = yield Git.checkSyncStatus(git);
        // get the latest releases and commits since
        const series = yield Rel.getCurrentSeries(git);
        // get package info
        const packageJson = yield PackageJson.read(process.cwd());
        if (packageJson === undefined) {
            // todo exception system
            throw new Error('could not find a package.json');
        }
        return {
            series,
            package: {
                name: packageJson.name,
            },
            githubRepo: {
                owner: githubRepoAddress.owner,
                name: githubRepoAddress.name,
                trunkBranch,
            },
            currentBranch: {
                name: branchesSummary.current,
                isTrunk: branchesSummary.current === trunkBranch,
                isDetatched: branchesSummary.detached,
                prs: prsByState,
                syncStatus,
            },
        };
    });
}
exports.scan = scan;
