"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This module handles the concerns of publishing. It handles interaction with
 * git tagging, pushing to the git origin, the package registry, etc.
 */
const promise_1 = __importDefault(require("simple-git/promise"));
const Pacman = __importStar(require("../lib/pacman"));
const defaultOpts = {
    gitTagVPrefix: false,
    skipNPM: false,
};
/**
 * Run the publishing process.
 *
 * 1. Change package.json version field to be new version.
 * 2. npm publish --tag next.
 * 3. discard package.json change.
 * 4. git tag {newVer}.
 * 5. git tag next.
 * 6. git push --tags.
 *
 */
function publish(release, givenOpts) {
    return __awaiter(this, void 0, void 0, function* () {
        const opts = Object.assign(Object.assign({}, defaultOpts), givenOpts);
        if (!opts.skipNPM) {
            // publish to the npm registry
            //
            // If we are using a script runner then publish with that same tool. Otherwise
            // default to using npm. The reason we need to do this is that problems occur
            // when mixing tools. For example `yarn run ...` will lead to a spawn of `npm
            // publish` failing due to an authentication error.
            const pacman = yield Pacman.create({ defualt: 'npm' });
            yield pacman.publish({ version: release.version, tag: release.distTag });
            console.log('published package to the npm registry');
            // When publishing it is sometimes desirable to update other dist tags to
            // point at the new version. For example "next" should never fall behind stable,
            // etc.
            //
            // todo parallel optimize?
            if (release.additiomalDistTags) {
                for (const distTag of release.additiomalDistTags) {
                    yield pacman.tag({ packageVersion: release.version, tagName: distTag });
                    console.log(`updated dist-tag "${distTag}" to point at this version`);
                }
            }
        }
        // While the fields of the package.json should not have changed, its
        // formatting, like indentation level, might have. We do not want to leave a
        // dirty working directoy on the user's system.
        //
        // TODO no invariant in system that checks that package.json was not modified
        // before beginning the publishing process. In other words we may be losing
        // user work here. This check should be in strict mode.
        const git = promise_1.default();
        yield git.checkout('package.json');
        console.log('reverted package.json changes now that publishing is done');
        // Tag the git commit
        //
        const versionTag = opts.gitTagVPrefix
            ? 'v' + release.version
            : release.version;
        yield git.addAnnotatedTag(versionTag, versionTag);
        console.log(`tagged this commit with ${versionTag}`);
        // Push the git commits
        //
        yield git.pushTags();
        console.log(`pushed tag to remote`);
    });
}
exports.publish = publish;
